/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
#define _HEAVY_CONTEXT_WSTD_SMTHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvControlCast.h"
#include "HvSignalVar.h"
#include "HvSignalEnvelope.h"
#include "HvControlIf.h"
#include "HvControlVar.h"
#include "HvControlBinop.h"
#include "HvMath.h"
#include "HvSignalLine.h"

class Heavy_WSTD_SMTHR : public HeavyContext {

 public:
  Heavy_WSTD_SMTHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_SMTHR();

  const char *getName() override { return "WSTD_SMTHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
        SMTHR = 0xD88E26F9, // Smthr
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_R7CIogQW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ZjXy2nDu_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_qdhdJ5WT_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_MuT58ROw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_DSHPL5F8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_rRoLKzGV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_pl1eM9eK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_kh55U0YW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_4EvwXXMr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_xPJvgHCW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_sEazKe5T_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_VCkmz5EZ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_3kbzmL4r_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_HIe6rIJL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_NXzuuRCO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_j2yvajvV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_VrgLbnyB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_9b4BUqlH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_2H0j35Yu_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Jy9CMUbn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_vatBUrSO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_3MwGjZ5N_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Y0KJjwGE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_CBsozFXx_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_eQKrVqAQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_4M5W26R4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_oldTT2LC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_MkOOM84C_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_fh1uWaw0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_JUvdAbgf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_kKxsnbh2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_0JHaGKZY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_PiOED468_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_Z8DnZ5gn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_BTsIfT59_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_cuppcelF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_OSL9ABCI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_IqBkhHmj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_hNximAJr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_itJ6TXI4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_vzVxE2qW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_ni2mRWbw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ChIhGvvb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_BaOocow3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_2CqjVJnN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Ocf4pT7M_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_UQLfz38X_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_mBwqg7uQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_yTn52zDk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_4FA34OYe_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_sUGYDQIv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_iKxcimuV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_KQwuRqsS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_0BuP5GD1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_MTwFXu2l_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_3kFJ58V7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_7sVxzlua_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_uSYNyB0L_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_Iog1tKxR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_LGhH2fl1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_gP0LKJCm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_cj16NC8S_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_Mkd4JYFm;
  SignalEnvelope sEnv_9b4BUqlH;
  SignalEnvelope sEnv_2H0j35Yu;
  SignalEnvelope sEnv_BTsIfT59;
  SignalEnvelope sEnv_cuppcelF;
  SignalLine sLine_un6HcuOd;
  SignalLine sLine_e0fZiG35;
  ControlBinop cBinop_R7CIogQW;
  ControlBinop cBinop_ZjXy2nDu;
  ControlBinop cBinop_qdhdJ5WT;
  ControlBinop cBinop_MuT58ROw;
  ControlBinop cBinop_pl1eM9eK;
  ControlBinop cBinop_4EvwXXMr;
  ControlBinop cBinop_xPJvgHCW;
  ControlBinop cBinop_sEazKe5T;
  ControlVar cVar_VCkmz5EZ;
  ControlVar cVar_HIe6rIJL;
  SignalVarf sVarf_v8WHyRd8;
  ControlVar cVar_NXzuuRCO;
  ControlIf cIf_j2yvajvV;
  ControlIf cIf_VrgLbnyB;
  ControlBinop cBinop_Jy9CMUbn;
  ControlBinop cBinop_3MwGjZ5N;
  ControlBinop cBinop_Y0KJjwGE;
  ControlBinop cBinop_4M5W26R4;
  ControlBinop cBinop_MkOOM84C;
  SignalVarf sVarf_xfvy3NrS;
  ControlVar cVar_0JHaGKZY;
  ControlIf cIf_PiOED468;
  ControlIf cIf_Z8DnZ5gn;
  ControlBinop cBinop_OSL9ABCI;
  ControlBinop cBinop_hNximAJr;
  ControlBinop cBinop_itJ6TXI4;
  ControlBinop cBinop_ChIhGvvb;
  ControlBinop cBinop_2CqjVJnN;
  ControlVar cVar_yTn52zDk;
  ControlBinop cBinop_4FA34OYe;
  SignalVarf sVarf_xZ2IFtay;
  SignalVarf sVarf_58TJchdF;
  SignalVarf sVarf_WkMyOJFj;
  SignalVarf sVarf_wbAZrSmb;
  ControlBinop cBinop_0BuP5GD1;
};

#endif // _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
